package io.ars.protocol;

import java.io.ObjectOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Vector;
import salsa.language.ActorReference;
import salsa.language.Message;
import salsa.language.ServiceFactory;

import io.IOTheater;
import io.core.agents.AgentFactory;
import io.core.agents.ProtocolAgent;
import io.core.language.AutonomousActorReference;
import io.core.protocol.IOPacket;
import io.core.protocol.TheaterLocator;
import io.core.protocol.JoinPacket;
import io.core.protocol.LeavePacket;
import io.core.peerserver.ServerHandler;
import io.ars.decision.ARSDecisionAgent;
import io.ars.profiling.ActorProfile;
import io.ars.profiling.ARSProfilingAgent;
	
public class ARSProtocolAgent implements ProtocolAgent {
	private Vector neighbors = new Vector();
	private int maxTimeToLive = 5;
	private TheaterLocator here = null;

	private Vector peerServers = new Vector();

	public ARSProtocolAgent() {
	}

	public TheaterLocator getLocation() { 
		if (here == null) {
			try {
				InetAddress IPHere = InetAddress.getLocalHost();
				String stringHere = "rmsp://" + IPHere.getHostAddress() + ":" + ServiceFactory.getTheater().getPort();
				here = new TheaterLocator(stringHere);
			} catch (UnknownHostException e) {
				System.err.println("Error getting localhost: " + e);
			}
		}
		return here; 
	}

	public void process(IOPacket received, Socket incoming) {
		ARSDecisionAgent decisionAgent = (ARSDecisionAgent)AgentFactory.getDecision();

                if (received instanceof ComputingCheck) {
                        try {
                                ObjectOutputStream out = new ObjectOutputStream(incoming.getOutputStream());
                                out.writeObject( new Boolean(decisionAgent.computing()) );
                        } catch (IOException e) {
                                System.err.println("Error on computing check: " + e);  
                        }
		} else if (received instanceof JoinPacket) {
			joinComputation(null);
		} else if (received instanceof LeavePacket) {
			leaveComputation();
		} else if (received instanceof ARSteal) {
			ARSteal steal = (ARSteal)received;

			ActorProfile candidate = decisionAgent.getBestCandidate(steal);

			if ( candidate != null ) {
				//if we should migrate an actor to the origin of this RandomSteal packet,
				//priority send that actor a message to do so.

				String migrationTarget = steal.getOrigin().toString() + candidate.getUAN().getIdentifier();

				ActorReference[] _targets = { new AutonomousActorReference(candidate.getUAN()) };
				String[] _methodNames = { "migrate" };
				Object[][] _arguments = { {migrationTarget} };
                
				candidate.prioritySend( new Message( null, _targets, _methodNames, _arguments ) );
			} else {
				propagateSteal(steal);
			}
		} else if (received instanceof Terminated) {
			decisionAgent.stealTerminated();
		} else if (received instanceof TheaterLocator) {
			addPeer( (TheaterLocator)received );
		}

		addPeer( received.getOrigin() );
   	}

	public Vector getValidNeighbors() {
                Vector validNeighbors = new Vector();
		boolean retval = false;

                for (int i = 0; i < neighbors.size(); i++) {
			try {
				ComputingCheck check = new ComputingCheck(here);
				retval = check.getResponse((TheaterLocator)neighbors.get(i)).booleanValue();
			} catch (IOException e) {
				neighbors.remove(i);
				i--;
			} catch (Exception e) {
				System.err.println("Error getting computing: " + e);
			}
                        if (retval) validNeighbors.add(neighbors.get(i));
                }
		return validNeighbors;
	}

	public void beginSteal(ARSteal steal) {
		if (neighbors.isEmpty()) {
			return;
		} else if ((steal.timeToLive < 0) || 
			  ( (neighbors.size() == 1) && ( ((TheaterLocator)neighbors.get(0)).equals(steal.getOrigin()) ))) {
			Terminated terminated = new Terminated(getLocation());			

			try {
				terminated.send(steal.getOrigin());
			} catch (Exception e) {
			}
			return;
		}

		TheaterLocator target = null;
		int i = 0;
		while ((target == null) || target.equals( steal.getOrigin())) {
			i = (int)java.lang.Math.random() * neighbors.size();
			target = (TheaterLocator)neighbors.get(i);
		}

		try {
			steal.send( target );
		} catch (Exception e) {
			neighbors.remove(i);
		}
	}

	private void propagateSteal(ARSteal steal) {
		if (steal.timeToLive > maxTimeToLive) steal.timeToLive = maxTimeToLive;
		steal.timeToLive--;
		beginSteal(steal);
	}

	public void joinComputation(Vector _peerServers) {
		((ARSDecisionAgent)AgentFactory.getDecision()).enterComputation();

		if (_peerServers != null) {
			peerServers.addAll(_peerServers);
		}


		if (!IOTheater.silent) {
			System.err.println("Connecting to PeerServers: ");

			for (int i = 0; i < peerServers.size(); i++) {
				System.err.println("\t" + (String)peerServers.get(i));
			}
		}
		ServerHandler serverHandler = new ServerHandler(peerServers, 10, AgentFactory.getProtocol().getLocation());
		serverHandler.start();
	}

        public void leaveComputation() {
		((ARSDecisionAgent)AgentFactory.getDecision()).leaveComputation();

		Vector actorProfiles = ((ARSProfilingAgent)AgentFactory.getProfiling()).getActorProfiles();
		if (actorProfiles.size() == 0) return;

                Vector validNeighbors = getValidNeighbors();
                //no valid neighbors to migrate the actors to,
                //so quit
                if (validNeighbors.size() == 0) return;

                int target = 0;
                for (int i = 0; i < actorProfiles.size(); i++) {
                        ActorProfile candidate = (ActorProfile)actorProfiles.get(i);

                        String migrationTarget = ((TheaterLocator)neighbors.get(target)).toString() +
                                                 candidate.getUAN().getIdentifier();

                        ActorReference[] _targets = { new AutonomousActorReference(candidate.getUAN()) };
                        String[] _methodNames = { "migrate" };
                        Object[][] _arguments = { {migrationTarget} };

                        candidate.prioritySend( new Message( null, _targets, _methodNames, _arguments ) );

                        target++;
                        if (target >= validNeighbors.size()) target = 0;
		}
	}


	private void addPeer(TheaterLocator peer) {
		if (peer == null) return;

		for (int i = 0; i < neighbors.size(); i++) {
			if (peer.equals( (TheaterLocator)neighbors.get(i) )) return;
		}
		neighbors.add(peer);
	}
}

