/*
The default agents are:
 	io.ars.decision.ARSDecisionAgent
	io.ars.profiling.ARSProfilingAgent
	io.ars.protocol.ARSProtocolAgent
Set by the system properties:
	decison
	profiling
	protocol
*/

package io.core.agents;

import salsa.language.exceptions.SalsaWrapperException;


public class AgentFactory {
	// Service implementations.
	private static DecisionAgent decisionAgent = null;
	private static ProfilingAgent profilingAgent = null;
	private static ProtocolAgent protocolAgent = null;

	/**
	 * Constructor for Service Factory
	 * This allows for different service implementations
	 * to be specified.
	 */
	public synchronized static void createFactory(DecisionAgent _decisionAgent, ProfilingAgent _profilingAgent,
	ProtocolAgent _protocolAgent) {
		protocolAgent = _protocolAgent;
		decisionAgent = _decisionAgent;
		profilingAgent = _profilingAgent;
	}

	public synchronized static void createFactory(String _decisionAgent, String _profilingAgent, 
						      String _protocolAgent) {

		if ((_decisionAgent != null) && (decisionAgent == null)) {
			try {
				decisionAgent = (DecisionAgent)Class.forName( _decisionAgent ).newInstance();
			} catch (Exception e) {
				throw new SalsaWrapperException( "Exception creating default decision agent.", e );
			}
		}

		if ((_profilingAgent != null) && (profilingAgent == null)) {
			try {
				profilingAgent = (ProfilingAgent)Class.forName( _profilingAgent ).newInstance();
			} catch (Exception e) {
				throw new SalsaWrapperException( "Exception creating default profiling agent.", e );
			}
		}

		if ((_protocolAgent != null) && (protocolAgent == null)) {
			try {
				protocolAgent = (ProtocolAgent)Class.forName( _protocolAgent ).newInstance();
			} catch (Exception e) {
				throw new SalsaWrapperException( "Exception creating default protocol agent.", e );
			}
		}
	}

	/**
	 * Returns an implementation of the decision agent. If no agent has been specified,
	 * it checks the system property "-Ddecision=<decisionagent>" and instantiates that
	 * implementation
	 */
	public synchronized static DecisionAgent getDecision() {
		if (decisionAgent == null) {
			// Check for the messagingService.
			String className = System.getProperty( "decision" );
			if (className == null) {
				className = "io.ars.decision.ARSDecisionAgent";
			}

			try {
				decisionAgent = (DecisionAgent)Class.forName( className ).newInstance();
			} catch (Exception e) {
				throw new SalsaWrapperException( "Exception creating default decision agent.", e );
			}
		}
		
		return decisionAgent;
	}


	/**
	 * Returns an implementation of the profiling agent. If no agent has been specified,
	 * it checks the system property "-Ddprofiling=<profilingagent>" and instantiates that
	 * implementation
	 */
	public synchronized static ProfilingAgent getProfiling() {
		if (profilingAgent == null) {
			// Check for the messagingService.
			String className = System.getProperty( "profiling" );
			if (className == null) {
				className = "io.ars.profiling.ARSProfilingAgent";
			}

			try {
				profilingAgent = (ProfilingAgent)Class.forName( className ).newInstance();
			} catch (Exception e) {
				throw new SalsaWrapperException( "Exception creating default profiling agent.", e );
			}
		}
		
		return profilingAgent;
	}


	/**
	 * Returns an implementation of the protocol agent. If no agent has been specified,
	 * it checks the system property "-Dprotocol=<protocolagent>" and instantiates that
	 * implementation
	 */
	public synchronized static ProtocolAgent getProtocol() {
		if (protocolAgent == null) {
			// Check for the messagingService.
			String className = System.getProperty( "protocol" );
			if (className == null) {
				className = "io.ars.protocol.ARSProtocolAgent";
			}

			try {
				protocolAgent = (ProtocolAgent)Class.forName( className ).newInstance();
			} catch (Exception e) {
				throw new SalsaWrapperException( "Exception creating default protocol agent.", e );
			}
		}
		
		return protocolAgent;
	}
}
