/** SALSA/World Wide Computer Project
 *  ObjectHandler recieves incoming objects
 *  and determines if they are actors or messages.
 *  upon recieving actors, it will start them up.
 *  messages get sent to the appropriate actor.
 *
 *  author Travis Desell
 */
package io.core.messaging;

import io.core.agents.AgentFactory;
import io.core.protocol.IOPacket;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.net.Socket;

import salsa.language.Actor;
import salsa.language.Message;
import salsa.messaging.ReceptionService;
import wwc.messaging.Handler;

public class IOHandler extends Handler implements ReceptionService {
	public void process(Socket incoming) {
		// Read in the object.
		Object received = null;
		ObjectInputStream stream = null;
                try {
                        stream = new ObjectInputStream( incoming.getInputStream() );
                        received = stream.readObject();
                } catch (ClassNotFoundException e) {
                        System.err.println( "Could not load class for " + received.getClass().getName() );
                        System.err.println( "  Is it in the theater CLASSPATH" );
                        System.err.println( e.getMessage() );
                        return;
                } catch (IOException e) {
                        System.err.println( "IOException occured reading object " + e.getMessage() );
                        return;
                }

                if (received instanceof IOPacket) {
			AgentFactory.getProtocol().process((IOPacket)received, incoming);
 		} else if (received instanceof Actor) {
			Actor incomingActor = (Actor) received;
			processActor(incomingActor, incoming);
		} else if (received instanceof Message) {  // just serializing data
			Message message = (Message) received;
			processMessage(message);
		}

		// Close the connection.
		try {
			incoming.close();
		} catch (IOException e) {
			System.err.println( "IOException occured closing connection" + e.getMessage() );
			return;
		}
	}
}
