package wwc.messaging;

import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.net.Socket;


import salsa.language.*;
import salsa.language.exceptions.*;
import salsa.naming.*;
import salsa.messaging.*;

/**
 * This class is the reference implementation of the MessagingService. The
 * service is implemented as a RMSP client.
 * 
 * This implementation uses the NamingService interface and the salsa.language.
 * ServiceFactory for UAN resolution.
 * 
 * @author stepha
 */
public class Messenger implements TransportService {
	/**
	 * This is the maximum number of times a message maybe forwarded before
	 * the name server is queried. Set by system property -DforwardThreshold=<n>
	 * The default value is 4.
	 */
	private int forwarderThreshold;

	/**
	 * Constructor for Messenger.
	 */
	public Messenger() {
		super();
		
		String thresh = System.getProperty( "forwardThreshold" );
		if (thresh != null) forwarderThreshold = Integer.parseInt( thresh );
		else forwarderThreshold = 4;
	}

	public void migrate(Actor state, UAL target) {
		Socket outgoing = null;
		try {
			outgoing = new Socket(target.getHost(), target.getPort());
			ObjectOutputStream outStream = new ObjectOutputStream(outgoing.getOutputStream());
			outStream.writeObject( state );
			outStream.flush();
		} catch (IOException e) {
			throw new SalsaWrapperException("Error migrating actor: ", e);
		}

		// Wait for the Acknowledgement that the actor is situated on the remote theater.
		try {
			RMSPAcknowledgement ack = (RMSPAcknowledgement) new ObjectInputStream( outgoing.getInputStream() ).readObject();
		} catch (Exception e) {
			throw new SalsaWrapperException( "Did not recieve RmspAcknowledgement. ", e);
		}

		try {
			outgoing.close();
		} catch (IOException e) {
			throw new SalsaWrapperException("Error closing socket: ", e);
		}
	}

	public void send(Message message, UAL target) {
//		System.err.print(":");

		// Determine the destination of this message.
		if (target == null) {
			UAN uan = message.getTarget().getUAN();

			if (uan != null) {
				Object entry = ServiceFactory.getTheater().getEntry(uan);
				if ((entry instanceof UAL) || (entry == null)) {
					// Resolve the location.
					target = ServiceFactory.getNaming().resolve( uan );
					message.setForwarded( 0 );
	
					ServiceFactory.getTheater().setEntry(uan, target);
				} else {
					((Actor)entry).send(message);
					return;
				}
			} else {
				System.err.println("UAN and UAL of message are null, cannot send message");
			}
		}

		// Update the number of times the message has been forwarded.
		message.setForwarded( message.getForwarded() + 1 );

		try {
			Socket outgoing = new Socket(target.getHost(), target.getPort());
			ObjectOutputStream outStream = new ObjectOutputStream(outgoing.getOutputStream());
			outStream.writeObject( message );
			outStream.flush();
			outStream.close();
			outgoing.close();
		} catch (IOException e) {
			throw new SalsaWrapperException("Error sending message: ", e);
		}
	}
}
